<?php
/**
 * Main Pro Class.
 *
 * @package Marketplace for WooCommerce
 *
 * @version 5.4.0
 */

namespace WkMarketplacePro\Includes;

use WkMarketplacePro\Includes;
use WkMarketplacePro\Helper;

defined( 'ABSPATH' ) || exit; // Exit if access directly.

if ( ! class_exists( 'WKMarketplace_Pro' ) ) {
	/**
	 * Marketplace Main Pro Class.
	 */
	final class WKMarketplace_Pro {
		/**
		 * Instance variable.
		 *
		 * @var object
		 */
		protected static $wk = null;

		/**
		 * General query helper.
		 *
		 * @var object
		 */
		protected $general_query;

		/**
		 * Dependency check property.
		 *
		 * @var bool $is_dependency_exists
		 */
		private $is_dependency_exists = true;

		/**
		 * The object is created from within the class itself only if the class has no instance.
		 *
		 * @return object|WKMarketplace|null
		 */
		public static function instance() {
			if ( is_null( self::$wk ) ) {
				self::$wk = new self();
			}

			return self::$wk;
		}

		/**
		 * Marketplace Constructor.
		 */
		public function __construct() {
			/**
			 * Load dependency classes like wkmp-functions.php
			 */
			$this->load_dependencies_support();

			/**
			 * Run dependency check to check if dependency available
			 */
			$this->do_dependency_check();

			if ( ! $this->is_dependency_exists ) {
				$this->manage_mp_lite_installation();
				add_action( 'admin_notices', array( $this, 'wkmp_show_lite_not_active_notice' ) );
			} else {
				$this->wkmp_init_hooks();
				$this->general_query = Helper\WKMP_General_Queries_Pro::get_instance();
			}
		}

		/**
		 * Load dependencies support.
		 *
		 * @return void
		 */
		public function load_dependencies_support() {
			/** Setting up MP Lite Dependency Classes */
			require_once __DIR__ . '/wkmp-includes/wkmp-functions.php';
		}

		/**
		 * DO dependency check.
		 *
		 * @return void
		 */
		public function do_dependency_check() {
			if ( ! wkmp_is_lite_installed() || ! wkmp_is_lite_active() || ! wkmp_is_min_lite_version_installed() || ! wkmp_is_woocommerce_active() ) {
				$this->is_dependency_exists = false;
			}
		}

		/**
		 * Hook into actions and filters.
		 *
		 * @return void|bool
		 */
		public function wkmp_init_hooks() {
			add_action( 'plugins_loaded', array( $this, 'wkmp_load_plugin' ), 11 );
			add_filter( 'wkmp_is_pro_module_disabled', '__return_false' );
			add_filter( 'wkmp_seller_wc_endpoints', array( $this, 'wkmp_pro_add_wc_seller_endpoints' ) );
		}

		/**
		 * Add Pro module endpoints.
		 *
		 * @param array $endpoints WC Seller endpoints.
		 *
		 * @return array
		 */
		public function wkmp_pro_add_wc_seller_endpoints( $endpoints = array() ) {
			$endpoints['shippings']          = get_option( '_wkmp_shipping_endpoint', 'seller-shippings' );
			$endpoints['separate-dashboard'] = get_option( '_wkmp_admin_dashboard_endpoint', 'separate-dashboard' );
			$endpoints['become_seller']      = get_option( '_wkmp_become_seller_endpoint', 'become-seller' );

			return $endpoints;
		}

		/**
		 * Load plugin.
		 *
		 * @return void
		 */
		public function wkmp_load_plugin() {
			Includes\WKMP_File_Handler_Pro::get_instance();
			add_action( 'before_woocommerce_init', array( $this, 'wkmp_pro_is_wc_hpos_compatible' ) );
		}

		/**
		 * Declares WooCommerce HPOS compatibility.
		 *
		 * @return void
		 */
		public function wkmp_pro_is_wc_hpos_compatible() {
			if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
				\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', WKMP_FILE, true );
			}
		}

		/**
		 * Show MP Lite not installed notice.
		 *
		 * @return void
		 */
		public function wkmp_show_lite_not_active_notice() {
			$screen = get_current_screen();
			if ( isset( $screen->parent_file ) && 'plugins.php' === $screen->parent_file && 'update' === $screen->id ) {
				return;
			}

			$plugin = defined( 'WKMP_LITE_PLUGIN_BASENAME' ) ? WKMP_LITE_PLUGIN_BASENAME : 'multi-vendor-marketplace-lite-for-woocommerce/multi-vendor-marketplace-lite-for-woocommerce.php';

			$message = '<p>' . wp_sprintf( /* Translators: %s MP Lite link. */ esc_html__( '%1$s plugin depends on the latest version of %2$s to work!', 'wk-marketplace' ), '<b>Marketplace for WooCommerce</b>', '<a href="https://wordpress.org/plugins/multi-vendor-marketplace-lite-for-woocommerce/" target="_blank">' . esc_html__( 'Marketplace Lite for WooCommerce', 'wk-marketplace' ) . '</a>' ) . '</p>';

			if ( ! wkmp_is_lite_installed() ) {
				if ( ! current_user_can( 'activate_plugins' ) ) {
					return;
				}

				$install_url    = 'javascript:void(0);';
				$id             = 'wkmp_lite_from_git';
				$lite_on_wprepo = $this->wkmp_pro_is_lite_on_wprepo();

				if ( $lite_on_wprepo ) {
					$install_url = wp_nonce_url( self_admin_url( 'update.php?action=install-plugin&plugin=multi-vendor-marketplace-lite-for-woocommerce' ), 'install-plugin_multi-vendor-marketplace-lite-for-woocommerce' );
					$id          = '';
				}

				$message .= '<p>' . wp_sprintf( '<a id="%s" href="%s" class="button-primary">%s</a>', $id, $install_url, __( 'Install "Marketplace Lite for WooCommerce" Now', 'wk-marketplace' ) ) . '</p>';
			}

			if ( wkmp_is_lite_installed() && ! wkmp_is_lite_active() ) {
				if ( ! current_user_can( 'install_plugins' ) ) {
					return;
				}

				$activation_url = wp_nonce_url( 'plugins.php?action=activate&amp;plugin=' . $plugin . '&amp;plugin_status=all&amp;paged=1&amp;s', 'activate-plugin_' . $plugin );
				$message       .= '<p>' . wp_sprintf( '<a href="%s" class="button-primary">%s</a>', $activation_url, __( 'Activate "Marketplace Lite for WooCommerce" Now', 'wk-marketplace' ) ) . '</p>';
			} elseif ( wkmp_is_lite_installed() && ! wkmp_is_min_lite_version_installed() ) {
				if ( ! current_user_can( 'update_plugins' ) ) {
					return;
				}

				$upgrade_link   = 'javascript:void(0);';
				$id             = 'wkmp_lite_from_git';
				$lite_on_wprepo = $this->wkmp_pro_is_lite_on_wprepo();

				if ( $lite_on_wprepo ) {
					$upgrade_link = wp_nonce_url( self_admin_url( 'update.php?action=upgrade-plugin&plugin=' ) . $plugin, 'upgrade-plugin_' . $plugin );
					$id           = '';
				}

				$message .= '<p>' . sprintf( '<a id="%s" href="%s" class="button-primary">%s</a>', $id, $upgrade_link, __( 'Update "Marketplace Lite for WooCommerce" Now', 'wk-marketplace' ) ) . '</p>';
			}
			echo '<div class="error"><p>' . wp_kses_post( $message ) . '</p></div>';
		}

		/**
		 * Check if Lite module is available on WP repo.
		 *
		 * @return bool
		 */
		public function wkmp_pro_is_lite_on_wprepo() {
			$lite_on_wprepo = false;

			$response      = wp_remote_get( 'https://wordpress.org/plugins/multi-vendor-marketplace-lite-for-woocommerce/' );
			$response_code = wp_remote_retrieve_response_code( $response );

			if ( 200 === $response_code ) {
				$response_body = wp_remote_retrieve_body( $response );

				if ( false === stripos( $response_body, 'Showing results for:' ) ) {
					$lite_on_wprepo = true;
				}
			}
			return $lite_on_wprepo;
		}

		/**
		 * Get the plugin url.
		 *
		 * @return string
		 */
		public function wkmp_pro_plugin_url() {
			return untrailingslashit( plugins_url( '/', WKMP_FILE ) );
		}

		/**
		 * Get the plugin path.
		 *
		 * @return string
		 */
		public function wkmp_pro_plugin_path() {
			return untrailingslashit( WKMP_PLUGIN_FILE );
		}

		/**
		 * MP Lite installation.
		 *
		 * @return void
		 */
		public function manage_mp_lite_installation() {
			add_action( 'admin_enqueue_scripts', array( $this, 'wkmp_pro_install_lite_scripts' ) );
			add_action( 'wp_ajax_wkmp_pro_install_lite', array( $this, 'wkmp_pro_get_lite_from_git' ) );
		}

		/**
		 * Pro install lite scripts.
		 *
		 * @return void
		 */
		public function wkmp_pro_install_lite_scripts() {
			$suffix = ( defined( 'WKWC_DEV' ) && true === WKWC_DEV ) ? '' : '.min';
			$path   = ( defined( 'WKWC_DEV' ) && true === WKWC_DEV ) ? 'build' : 'dist';

			$ajax_obj = array(
				'ajaxUrl'   => admin_url( 'admin-ajax.php' ),
				'ajaxNonce' => wp_create_nonce( 'wkmp-lite-admin-nonce' ),
			);

			wp_enqueue_script( 'wkmp_lite_scripts', WKMP_PLUGIN_URL . 'assets/' . $path . '/admin/js/wkmp-lite-install' . $suffix . '.js', array(), WKMP_SCRIPT_VERSION, true );

			wp_localize_script(
				'wkmp_lite_scripts',
				'wkmpLiteObj',
				array(
					'ajax' => $ajax_obj,
				)
			);
		}

		/**
		 * Installing MP Lite plugin from git.
		 */
		public function wkmp_pro_get_lite_from_git() {
			if ( check_ajax_referer( 'wkmp-lite-admin-nonce', 'wkmp_nonce', false ) ) {
				$result = array(
					'success'  => false,
					'btn_text' => esc_html__( 'Could not install', 'wk-marketplace' ),
					'message'  => wp_sprintf( /* translators: %1$s: Git link open tag, %2$s: closing tag.  */ esc_html__( 'There are some issue, please again later or click %1$s here %2$s to download it manually.', 'wk-marketplace' ), '<a href="https://github.com/webkul/multi-vendor-marketplace-lite-for-woocommerce/raw/release/multi-vendor-marketplace-lite-for-woocommerce.zip">', '</a>' ),
				);

				$dir = dirname( dirname( __DIR__ ) );

				try {
					$f = file_put_contents( 'multi-vendor-marketplace-lite-for-woocommerce.zip', fopen( esc_url( 'https://github.com/webkul/multi-vendor-marketplace-lite-for-woocommerce/raw/release/multi-vendor-marketplace-lite-for-woocommerce.zip' ), 'r' ), LOCK_EX );

					if ( empty( $f ) ) {
						$result['btn_text'] = esc_html__( 'Could not write to files.', 'wk-marketplace' );
					} else {
						$zip = new \ZipArchive();
						$res = $zip->open( 'multi-vendor-marketplace-lite-for-woocommerce.zip' );

						if ( true === $res ) {
							$zip->extractTo( $dir );
							$zip->close();

							$result['success']  = true;
							$result['btn_text'] = esc_html__( 'Installed successfully...', 'wk-marketplace' );
						}
					}
				} catch ( \Exception $e ) {
					if ( ! empty( $e->getMessage() ) ) {
						$result['message'] = $e->getMessage();
					}
				}

				wp_send_json( $result );
			}
		}
	}
}
