<?php
/**
 * Seller back end Transactions list class.
 *
 * @package Marketplace for WooCommerce
 *
 * @version 5.0.0
 */

namespace WkMarketplacePro\Separate_Dashboard;

defined( 'ABSPATH' ) || exit; // Exit if access directly.

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

if ( ! class_exists( 'WKMP_Seller_Backend_Transaction_List' ) ) {
	/**
	 * Class WKMP_Seller_Backend_Transaction_List
	 *
	 * @package WkMarketplace\Separate_Dashboard
	 */
	class WKMP_Seller_Backend_Transaction_List extends \WP_List_Table {
		/**
		 * Instance variable
		 *
		 * @var $instance
		 */
		protected static $instance = null;

		/**
		 * WKMP_Seller_Backend_Transaction_List constructor.
		 */
		public function __construct() {
			parent::__construct(
				array(
					'singular' => esc_html__( 'Transaction', 'wk-marketplace' ),
					'plural'   => esc_html__( 'Transactions', 'wk-marketplace' ),
					'ajax'     => false,
				)
			);
		}

		/**
		 * Ensures only one instance of this class is loaded or can be loaded.
		 *
		 * @return object
		 */
		public static function get_instance() {
			if ( ! static::$instance ) {
				static::$instance = new self();
			}
			return static::$instance;
		}

		/**
		 * Prepare items.
		 */
		public function prepare_items() {
			$columns   = $this->get_columns();
			$sortable  = $this->get_sortable_columns();
			$hidden    = $this->get_hidden_columns();
			$seller_id = get_current_user_id();

			$current_page = $this->get_pagenum();
			$per_page     = $this->get_items_per_page( 'order_per_page', 10 );
			$data         = $this->table_data( $seller_id, $current_page, $per_page );

			$transactions = empty( $data['transactions'] ) ? array() : $data['transactions'];
			$total_items  = empty( $data['total_count'] ) ? 0 : intval( $data['total_count'] );

			$this->_column_headers = array( $columns, $hidden, $sortable );

			$total_pages = ceil( $total_items / $per_page );

			$this->set_pagination_args(
				array(
					'total_items' => $total_items,
					'total_pages' => $total_pages,
					'per_page'    => $per_page,
				)
			);

			$this->items = $transactions;
		}

		/**
		 * Get hidden columns.
		 *
		 * @return array
		 */
		public function get_hidden_columns() {
			return array();
		}

		/**
		 * Ger columns.
		 *
		 * @return array
		 */
		public function get_columns() {
			return array(
				'transaction_id'   => esc_html__( 'Transaction ID', 'wk-marketplace' ),
				'transaction_date' => esc_html__( 'Date', 'wk-marketplace' ),
				'amount'           => esc_html__( 'Amount', 'wk-marketplace' ),
				'action'           => esc_html__( 'Action', 'wk-marketplace' ),
			);
		}

		/**
		 * Get sortable columns.
		 *
		 * @return array[]
		 */
		public function get_sortable_columns() {
			return array(
				'transaction_id'   => array( 'transaction_id', true ),
				'transaction_date' => array( 'transaction_date', true ),
				'amount'           => array( 'amount', true ),
			);
		}

		/**
		 * Columns default.
		 *
		 * @param array  $item Product item.
		 * @param string $column_name Column name.
		 *
		 * @return string
		 */
		public function column_default( $item, $column_name ) {
			switch ( $column_name ) {
				case 'transaction_id':
				case 'transaction_date':
				case 'amount':
				case 'action':
					return $item[ $column_name ];
				default:
					return '-';
			}
		}

		/**
		 * Table data.
		 *
		 * @param int $user_id Seller id.
		 * @param int $page_no Current page.
		 * @param int $limit Limit.
		 *
		 * @return array
		 */
		private function table_data( $user_id, $page_no, $limit ) {
			$transactions = $this->wkmp_get_seller_dashboard_transactions( $user_id, $limit, $page_no );
			$data         = array();

			if ( $transactions['transaction'] ) {
				foreach ( $transactions['transaction'] as $value ) {
					$transaction_id = isset( $value['id'] ) ? $value['id'] : 0;
					$data[]         = array(
						'transaction_id'   => isset( $value['transaction_id'] ) ? $value['transaction_id'] : '-',
						'transaction_date' => isset( $value['transaction_date'] ) ? get_date_from_gmt( $value['transaction_date'] ) : '-',
						'amount'           => isset( $value['amount'] ) ? wc_price( $value['amount'] ) : '-',
						'action'           => '<a href="' . admin_url( 'admin.php?page=seller-transaction&action=view&tid=' . $transaction_id ) . '" class="button button-primary">' . __( 'View', 'wk-marketplace' ) . '</a>',
					);
				}
			}

			return array(
				'total_count'  => $transactions['total_count'],
				'transactions' => $data,
			);
		}

		/**
		 * Get transactions.
		 *
		 * @param int $seller_id Seller id.
		 * @param int $limit Limit.
		 * @param int $current_page Current page.
		 *
		 * @return array
		 */
		public function wkmp_get_seller_dashboard_transactions( $seller_id, $limit, $current_page = 1 ) {
			global $wpdb;

			$search     = \WK_Caching::wk_get_request_data( 's' );
			$orderby    = \WK_Caching::wk_get_request_data( 'orderby', array( 'default' => 'transaction_id' ) );
			$sort_order = \WK_Caching::wk_get_request_data( 'order', array( 'default' => 'desc' ) );
			$offset     = ( $current_page - 1 ) * $limit;

			$all_transactions = $wpdb->get_results( $wpdb->prepare( "SELECT count(id) as total_transactions FROM {$wpdb->prefix}seller_transaction WHERE seller_id=%d", $seller_id ), ARRAY_A );

			$all_transactions   = ( is_iterable( $all_transactions ) && count( $all_transactions ) > 0 ) ? $all_transactions['0'] : array();
			$total_transactions = empty( $all_transactions['total_transactions'] ) ? 0 : intval( $all_transactions['total_transactions'] );

			$query = $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}seller_transaction WHERE seller_id=%d", $seller_id );

			if ( ! empty( $search ) ) {
				$query .= $wpdb->prepare( ' and transaction_id like %s', '%' . $wpdb->esc_like( $search ) . '%' );
			}

			$query .= $wpdb->prepare( ' ORDER BY %1s %2s LIMIT %d, %d', esc_sql( $orderby ), esc_sql( $sort_order ), esc_sql( $offset ), esc_sql( $limit ) );

			$transactions = $wpdb->get_results( $query, ARRAY_A );

			return array(
				'total_count' => $total_transactions,
				'transaction' => $transactions,
			);
		}
	}
}
