<?php
/**
 * Front template Functions.
 *
 * @package Marketplace for WooCommerce
 * @version 5.0.0
 */

namespace WkMarketplacePro\Templates\Front;

use WkMarketplacePro\Templates\Front\Customer;
use WkMarketplacePro\Templates\Front\Seller\Shipping;

defined( 'ABSPATH' ) || exit; // Exit if access directly.

if ( ! class_exists( 'WKMP_Front_Template_Functions_Pro' ) ) {
	/**
	 * Front template functions class.
	 *
	 * Class WKMP_Front_Template_Functions
	 *
	 * @package WkMarketplace\Templates\Front
	 */
	class WKMP_Front_Template_Functions_Pro {
		/**
		 * Instance variable
		 *
		 * @var $instance
		 */
		protected static $instance = null;

		/**
		 * WKMP_Front_Template_Functions constructor.
		 */
		public function __construct() {
		}

		/**
		 * Ensures only one instance of this class is loaded or can be loaded.
		 *
		 * @return object
		 */
		public static function get_instance() {
			if ( ! static::$instance ) {
				static::$instance = new self();
			}
			return static::$instance;
		}

		/**
		 * Callback method for Add new query var.
		 *
		 * @param array $vars Variables.
		 *
		 * @hooked 'query_vars' action hook.
		 *
		 * @return array
		 */
		public function wkmp_pro_add_query_vars( $vars ) {
			global $wkmarketplace;
			$customer_id = get_current_user_id();

			if ( $wkmarketplace->wkmp_user_is_customer( $customer_id ) ) {
				$become_seller = get_option( '_wkmp_become_seller_endpoint', 'become-seller' );
				$vars[]        = $become_seller;
			}

			return $vars;
		}

		/**
		 * Callback method for Insert new endpoints into the My Account menu.
		 *
		 * @param array $items menu items.
		 *
		 * @hooked 'woocommerce_account_menu_items' filter hook.
		 *
		 * @return array
		 */
		public function wkmp_pro_new_menu_items( $items ) {
			global $wkmarketplace;
			$items       = is_array( $items ) ? $items : array();
			$customer_id = get_current_user_id();
			$logout      = '';

			if ( isset( $items['customer-logout'] ) ) {
				// Remove the logout menu item.
				$logout = $items['customer-logout'];
				unset( $items['customer-logout'] );
			}

			// Insert Become a seller endpoint.
			if ( $wkmarketplace->wkmp_user_is_customer( $customer_id ) ) {
				$become_seller           = get_option( '_wkmp_become_seller_endpoint', 'become-seller' );
				$become_title            = get_option( '_wkmp_become_seller_endpoint_name', esc_html__( 'Become a Seller', 'wk-marketplace' ) );
				$items[ $become_seller ] = $become_title;
			}

			if ( ! empty( $logout ) ) {
				// Insert back the logout item.
				$items['customer-logout'] = $logout;
			}

			return $items;
		}

		/**
		 * Callback method for Set endpoint title
		 *
		 * @param string $title Title.
		 *
		 * @hooked 'the_title' Filter hook.
		 *
		 * @return string $title Endpoint title.
		 */
		public function wkmp_pro_endpoint_title( $title ) {
			global $wp_query, $wkmarketplace;

			$customer_id               = get_current_user_id();
			$become_seller             = get_option( '_wkmp_become_seller_endpoint', 'become-seller' );
			$is_become_seller_endpoint = isset( $wp_query->query_vars[ $become_seller ] );

			if ( $is_become_seller_endpoint && $wkmarketplace->wkmp_user_is_customer( $customer_id ) && ! is_admin() && is_main_query() && in_the_loop() && is_account_page() ) {
				$title = get_option( '_wkmp_become_seller_endpoint_name', esc_html__( 'Become a Seller', 'wk-marketplace' ) );
				remove_filter( 'the_title', array( $this, 'wkmp_pro_endpoint_title' ) );

				return $title;
			}

			// Seller endpoints title.
			if ( in_the_loop() && $wkmarketplace->wkmp_user_is_seller( $customer_id ) && ! is_admin() && is_main_query() && is_account_page() ) {
				$seller_endpoint = get_option( '_wkmp_shipping_endpoint', 'seller-shippings' );

				if ( isset( $wp_query->query_vars[ $seller_endpoint ] ) ) {
					$title = get_option( '_wkmp_shipping_endpoint_name', esc_html__( 'Shippings', 'wk-marketplace' ) );
					remove_filter( 'the_title', array( $this, 'wkmp_pro_endpoint_title' ) );

					return $title;
				}
			}

			return $title;
		}

		/**
		 * Callback method for display Become a seller form.
		 *
		 * @return void
		 */
		public function wkmp_pro_become_seller_endpoint_content() {
			new Customer\WKMP_Customer_Become_Seller( get_current_user_id() );
		}

		/**
		 * Show terms and conditions checkbox on seller sign up and become a seller form.
		 */
		public function wkmp_pro_maybe_show_signup_term_checkbox() {
			$seller_term_page_id = get_option( 'wkmp_seller_terms_page_id', 0 );

			if ( ! empty( $seller_term_page_id ) ) {
				$text      = apply_filters( 'wkmp_seller_signup_checkbox_text', esc_html__( 'I have read and agree to the seller signup', 'wk-marketplace' ), $seller_term_page_id );
				$link_text = apply_filters( 'wkmp_seller_signup_checkbox_link_text', esc_html__( 'terms and conditions', 'wk-marketplace' ), $seller_term_page_id );
				?>
			<p class="form-row validate-required">
				<label class="woocommerce-form__label woocommerce-form__label-for-checkbox checkbox">
				<input value="yes" type="checkbox" class="woocommerce-form__input woocommerce-form__input-checkbox input-checkbox" name="wkmp_seller_signup_term_accept" id="wkmp_seller_signup_term_accept">
					<span class="wkmp-seller-signup-terms"><?php echo esc_html( $text ); ?> <a href="<?php echo esc_url( get_permalink( $seller_term_page_id ) ); ?>" class="woocommerce-terms-and-conditions-link" target="_blank"><?php echo esc_html( $link_text ); ?></a></span>&nbsp;<abbr class="required" title="required">*</abbr>
				</label>
			</p>
				<?php
			}
		}

		/**
		 * Show Become a seller button on seller page for customer.
		 *
		 * @param int $customer_id Customer id.
		 */
		public function wkmp_pro_show_become_seller_button( $customer_id ) {
			global $wkmarketplace;
			if ( $wkmarketplace->wkmp_user_is_customer( $customer_id ) ) {
				$become_title  = get_option( '_wkmp_become_seller_endpoint_name', esc_html__( 'Become a Seller', 'wk-marketplace' ) );
				$become_seller = get_option( '_wkmp_become_seller_endpoint', 'become-seller' );
				?>
				<a href="<?php echo esc_url( get_permalink( get_option( 'woocommerce_myaccount_page_id' ) ) . $become_seller ); ?>" title="<?php echo esc_attr( $become_title ); ?>" class="wkmp-become-seller button"><?php echo esc_html( $become_title ); ?></a>
				<?php
			}
		}

		/**
		 * Templates to use in js.
		 *
		 * @return void
		 */
		public function wkmp_pro_front_footer_templates() {
			?>
			<script id="tmpl-wkmp_field_empty" type="text/html">
				<div class="wkmp-error">
					<p><?php esc_html_e( 'This is required data.', 'wk-marketplace' ); ?></p>
				</div>
			</script>
			<?php
			$show_info = \WK_Caching::wk_get_request_data( 'wkmodule_info', array( 'filter' => 'int' ) );
			$show_info = empty( $show_info ) ? 0 : intval( $show_info );

			if ( 200 === $show_info ) {
				?>
			<input type="hidden" data-lwdt="202401242045" data-wkmp_version="<?php echo esc_attr( MARKETPLACE_VERSION ); ?>">
				<?php
			}
		}
	}
}
